package cn.com.taiji.esshipservice.listener.hlx.fusion;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import cn.com.taiji.esshipservice.utils.HttpsDownloadUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.client.config.RequestConfig;
import org.elasticsearch.action.DocWriteResponse;
import org.elasticsearch.action.bulk.BulkItemResponse;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.bulk.BulkResponse;
import org.elasticsearch.action.delete.DeleteResponse;
import org.elasticsearch.action.index.IndexResponse;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.update.UpdateRequest;
import org.elasticsearch.action.update.UpdateResponse;
import org.elasticsearch.client.HttpAsyncResponseConsumerFactory;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.core.TimeValue;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.*;

import static cn.com.taiji.esshipservice.constant.ShipHisTrackIndexConstants.*;

/**
 * @author xhl
 * @date 2023/2/24
 */
@Component
@Slf4j
public class EsHkImagesDumpS3 {

    @Autowired
    private RestHighLevelClient client;

    private static final RequestOptions COMMON_OPTIONS;

    @Autowired
    private HttpsDownloadUtils httpsDownloadUtils;

    static {
        RequestConfig requestConfig = RequestConfig.custom()
                .setConnectTimeout(5000)
                .setSocketTimeout(6000)
                .build();
        RequestOptions.Builder builder = RequestOptions.DEFAULT.toBuilder();
        builder.setRequestConfig(requestConfig).setHttpAsyncResponseConsumerFactory(
                // 设置查询内容大小限制,默认100 * 1024 * 1024
                new HttpAsyncResponseConsumerFactory.HeapBufferedResponseConsumerFactory(5*1024 * 1024 * 1024)
        );
        COMMON_OPTIONS = builder.build();
    }

    ExecutorService threadPool = new ThreadPoolExecutor( 15,35,10L, TimeUnit.SECONDS, new LinkedBlockingDeque<>(10), Executors.defaultThreadFactory(), new ThreadPoolExecutor.DiscardPolicy());

    @Scheduled(fixedRate=60*60*1000)
    public void scheduled() {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR_OF_DAY, calendar.get(Calendar.HOUR_OF_DAY) -2);
        Date time1 = calendar.getTime();
        Calendar calendar2 = Calendar.getInstance();
        calendar2.set(Calendar.HOUR_OF_DAY, calendar2.get(Calendar.HOUR_OF_DAY) -1);
        Date time2 = calendar2.getTime();
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSXXX");
        String startTime = sdf1.format(time1);
        String endTime = sdf1.format(time2);
        threadPool.execute(()->{
            try {
                imagesDumpS3Car(startTime,endTime);
            } catch (IOException e) {
                e.printStackTrace();
            }
        });
        threadPool.execute(()->{
            try {
                imagesDumpS3Face(startTime,endTime);
            } catch (IOException e) {
                e.printStackTrace();
            }
        });
//        imagesDumpS3Car(startTime,endTime);
//        imagesDumpS3Face(startTime,endTime);
    }

    public void imagesDumpS3Car(String startTime,String endTime) throws IOException {
        //设置查询索引
        SearchRequest searchRequest = new SearchRequest(INDEX_SEAT_HIK_CAPTURE_CAR_TRACKS);
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        BoolQueryBuilder boolQueryBuilder= QueryBuilders.boolQuery();
        boolQueryBuilder.filter(QueryBuilders.rangeQuery("captureTime").gte(startTime).lte(endTime));
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder
                .trackTotalHits(true)
                .size(1000000);
        searchRequest.source(searchSourceBuilder);
        SearchResponse search = null;
        try {
            search = client.search(searchRequest, COMMON_OPTIONS);
        } catch (IOException e) {
            e.printStackTrace();
        }
        SearchHits searchHits = search.getHits();
        SearchHit[] hits = searchHits.getHits();
        BulkRequest bulkRequest = new BulkRequest();
        for (SearchHit hit : hits) {
            Map<String, Object> map = hit.getSourceAsMap();
            Map<String, Object> maps = Collections.synchronizedMap(new HashMap<>());
            Object plateImagePath = map.get("plateImagePath");
            if (null != plateImagePath   && !plateImagePath.equals("")){
                maps.put("plateImagePath", httpsDownloadUtils.downloadCarFileHttps(plateImagePath.toString()));
            }
            Object targetPicUrl = map.get("targetPicUrl");
            if (null != targetPicUrl  && !targetPicUrl.equals("")){
                maps.put("targetPicUrl", httpsDownloadUtils.downloadCarFileHttps(targetPicUrl.toString()));
            }
            if (!maps.isEmpty()) {
                bulkRequest.add(new UpdateRequest(INDEX_SEAT_HIK_CAPTURE_CAR_TRACKS, hit.getId()).doc(maps));
            }
            if (bulkRequest.requests().size() > 1000){
                try {
                    BulkResponse bulk = client.bulk(bulkRequest, RequestOptions.DEFAULT);
                    log.info("车辆执行修改::破千修改");
//                    for (BulkItemResponse bulkItemResponse : bulk) {
//                        DocWriteResponse response = bulkItemResponse.getResponse();
//                        switch (bulkItemResponse.getOpType()){
//                            case UPDATE:
//                                UpdateResponse updateResponse = (UpdateResponse)response;
//                                log.info("车辆修改"+updateResponse.getResult());
//                                break;
//                        }
//                    }
                    bulkRequest.requests().clear();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        try {
            if (bulkRequest.requests().size()>0){
                BulkResponse bulk = client.bulk(bulkRequest, RequestOptions.DEFAULT);
                log.info("车辆执行修改:最终修改");
//                for (BulkItemResponse bulkItemResponse : bulk) {
//                    DocWriteResponse response = bulkItemResponse.getResponse();
//                    switch (bulkItemResponse.getOpType()){
//                        case UPDATE:
//                            UpdateResponse updateResponse = (UpdateResponse)response;
//                            log.info("车辆修改"+updateResponse.getResult());
//                            break;
//                    }
//                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void imagesDumpS3Face(String startTime,String endTime) throws IOException {
        //设置查询索引
        SearchRequest searchRequest = new SearchRequest(INDEX_SEAT_HIK_CAPTURE_FACE_TRACKS);
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        BoolQueryBuilder boolQueryBuilder= QueryBuilders.boolQuery();
        boolQueryBuilder.filter(QueryBuilders.rangeQuery("captureTime").gte(startTime).lte(endTime));
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder
                .trackTotalHits(true)
                .size(1000000);
//                .timeout(TimeValue.timeValueHours(1L))
//                .timeout(TimeValue.timeValueMinutes(30L))
//                .timeout(TimeValue.timeValueSeconds(500L));
        searchRequest.source(searchSourceBuilder);
        SearchResponse search = null;
        try {
            search = client.search(searchRequest, COMMON_OPTIONS);
        } catch (IOException e) {
            e.printStackTrace();
        }
        SearchHits searchHits = search.getHits();
        SearchHit[] hits = searchHits.getHits();
        BulkRequest bulkRequest = new BulkRequest();
        for (SearchHit hit : hits) {
            Map<String, Object> map = hit.getSourceAsMap();
            Map<String, Object> maps = Collections.synchronizedMap(new HashMap<>());
            Object bkgUrl = map.get("bkgUrl");
            if (null != bkgUrl && !bkgUrl.equals("")){
                maps.put("bkgUrl", httpsDownloadUtils.downloadFaceFileHttps(bkgUrl.toString()));
            }
            Object facePicUrl = map.get("facePicUrl");
            if (null != facePicUrl && !facePicUrl.equals("")){
//                Boolean num = false;
//                StringTokenizer tokenizer = new StringTokenizer(facePicUrl.toString(),"/");
//                /** 判断枚举对象中是否还有数据 **/
//                while (tokenizer.hasMoreElements()){
//                    /** 返回从当前位置到下一个分隔符的字符串 **/
//                    if (tokenizer.nextToken().equals("74.10.28.62:81")){
//                        num =true;
//                        return;
//                    }
//                }
//                if (num == false){
                maps.put("facePicUrl", httpsDownloadUtils.downloadFaceFileHttps(facePicUrl.toString()));
//                }
            }
            if (!maps.isEmpty()) {
                bulkRequest.add(new UpdateRequest(INDEX_SEAT_HIK_CAPTURE_FACE_TRACKS, hit.getId()).doc(maps));
            }
            if (bulkRequest.requests().size() > 100){
                try {
                    BulkResponse bulk = client.bulk(bulkRequest, RequestOptions.DEFAULT);
                    log.info("人员执行修改:破百修改");
//                    for (BulkItemResponse bulkItemResponse : bulk) {
//                        DocWriteResponse response = bulkItemResponse.getResponse();
//                        switch (bulkItemResponse.getOpType()){
//                            case UPDATE:
//                                UpdateResponse updateResponse = (UpdateResponse)response;
//                                log.info("人员修改"+updateResponse.getResult());
//                                break;
//                        }
//                    }
                    bulkRequest.requests().clear();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        try {
            if (bulkRequest.requests().size()>0){
                BulkResponse bulk = client.bulk(bulkRequest, RequestOptions.DEFAULT);
                log.info("人员执行修改:最终修改");
//                for (BulkItemResponse bulkItemResponse : bulk) {
//                    DocWriteResponse response = bulkItemResponse.getResponse();
//                    switch (bulkItemResponse.getOpType()){
//                        case UPDATE:
//                            UpdateResponse updateResponse = (UpdateResponse)response;
//                            log.info("人员修改"+updateResponse.getResult());
//                            break;
//                    }
//                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}

